
-- ff_waterpolo.lua

-----------------------------------------------------------------------------
-- includes
-----------------------------------------------------------------------------
IncludeScript("base_teamplay");

-----------------------------------------------------------------------------
-- global overrides
-----------------------------------------------------------------------------
POINTS_PER_GOAL = 10
POINTS_PER_GOALIE_RETURN = 5
POINTS_PER_GOALIE_ATTACK = 1
BALL_RETURN_TIME = 10
BALL_THROW_SPEED = 512




-- precache (sounds)
function precache()
	PrecacheSound("misc.bizwarn")
	PrecacheSound("misc.bloop")
	PrecacheSound("misc.buzwarn")
	PrecacheSound("misc.dadeda")
	PrecacheSound("misc.deeoo")
	PrecacheSound("misc.doop")
	PrecacheSound("misc.woop")

	-- Unagi Power!  Unagi!
	PrecacheSound("misc.unagi")
end




function startup()

	-- set up team limits (blue & red are unlimited, while yellow and green get 1 goalie each)
	SetPlayerLimit( Team.kBlue, 0 )
	SetPlayerLimit( Team.kRed, 0 )
	SetPlayerLimit( Team.kYellow, 1 )
	SetPlayerLimit( Team.kGreen, 1 )

--	SetTeamName( Team.kBlue, "#FF_WATERPOLO_TEAMNAME_BLUE" )
--	SetTeamName( Team.kRed, "#FF_WATERPOLO_TEAMNAME_RED" )
--	SetTeamName( Team.kYellow, "#FF_WATERPOLO_TEAMNAME_YELLOW" )
--	SetTeamName( Team.kGreen, "#FF_WATERPOLO_TEAMNAME_GREEN" )

	SetTeamName( Team.kBlue, "Blue" )
	SetTeamName( Team.kRed, "Red" )
	SetTeamName( Team.kYellow, "Blue Goalie" )
	SetTeamName( Team.kGreen, "Red Goalie" )

	local team = GetTeam(Team.kBlue)
	team:SetAllies( Team.kYellow )
	team:SetClassLimit( Player.kScout, 0 )
	team:SetClassLimit( Player.kSniper, -1 )
	team:SetClassLimit( Player.kSoldier, 0 )
	team:SetClassLimit( Player.kDemoman, 0 )
	team:SetClassLimit( Player.kMedic, 0 )
	team:SetClassLimit( Player.kHwguy, 0 )
	team:SetClassLimit( Player.kPyro, 0 )
	team:SetClassLimit( Player.kSpy, 0 )
	team:SetClassLimit( Player.kEngineer, -1 )
	team:SetClassLimit( Player.kCivilian, -1 )

	team = GetTeam(Team.kRed)
	team:SetAllies( Team.kGreen )
	team:SetClassLimit( Player.kScout, 0 )
	team:SetClassLimit( Player.kSniper, -1 )
	team:SetClassLimit( Player.kSoldier, 0 )
	team:SetClassLimit( Player.kDemoman, 0 )
	team:SetClassLimit( Player.kMedic, 0 )
	team:SetClassLimit( Player.kHwguy, 0 )
	team:SetClassLimit( Player.kPyro, 0 )
	team:SetClassLimit( Player.kSpy, 0 )
	team:SetClassLimit( Player.kEngineer, -1 )
	team:SetClassLimit( Player.kCivilian, -1 )

	team = GetTeam(Team.kYellow)
	team:SetAllies( Team.kBlue )
	team:SetClassLimit( Player.kScout, -1 )
	team:SetClassLimit( Player.kSniper, -1 )
	team:SetClassLimit( Player.kSoldier, -1 )
	team:SetClassLimit( Player.kDemoman, -1 )
	team:SetClassLimit( Player.kMedic, -1 )
	team:SetClassLimit( Player.kHwguy, -1 )
	team:SetClassLimit( Player.kPyro, -1 )
	team:SetClassLimit( Player.kSpy, -1 )
	team:SetClassLimit( Player.kEngineer, -1 )
	team:SetClassLimit( Player.kCivilian, 0 )

	team = GetTeam(Team.kGreen)
	team:SetAllies( Team.kRed )
	team:SetClassLimit( Player.kScout, -1 )
	team:SetClassLimit( Player.kSniper, -1 )
	team:SetClassLimit( Player.kSoldier, -1 )
	team:SetClassLimit( Player.kDemoman, -1 )
	team:SetClassLimit( Player.kMedic, -1 )
	team:SetClassLimit( Player.kHwguy, -1 )
	team:SetClassLimit( Player.kPyro, -1 )
	team:SetClassLimit( Player.kSpy, -1 )
	team:SetClassLimit( Player.kEngineer, -1 )
	team:SetClassLimit( Player.kCivilian, 0 )

end

function remove_hud_items(ball, player)

	RemoveHudItem( player, ball:GetName() )

	local team = player:GetTeamId()

	if (team == Team.kBlue) then
		RemoveHudItemFromAll( "ball-icon-blue" )
	elseif (team == Team.kRed) then
		RemoveHudItemFromAll( "ball-icon-red" )
	end

end

function player_ondamage( player, damageinfo )
	ConsoleToAll( "player_ondamage" )
	-- goalies are invincible
	if player:GetTeamId() == Team.kYellow or player:GetTeamId() == Team.kGreen then
		damageinfo:SetDamage(0)
	-- goalies also have Unagi Power!  Unagi!
	else
		local attacker = damageinfo:GetAttacker()
		if IsPlayer( attacker ) then
			attacker = CastToPlayer( attacker )
			if attacker:GetTeamId() == Team.kYellow or attacker:GetTeamId() == Team.kGreen then
				ConsoleToAll( attacker:GetName() .. " has Unagi Power!  Unagi!" )
				BroadCastSound ( "misc.unagi" )
				attacker:AddFortPoints( POINTS_PER_GOALIE_ATTACK * 10, "#FF_FORTPOINTS_GOALIE_ATTACK" )
				damageinfo:ScaleDamage(69)
			end
		end
	end
end




-- ball information
base_ball = info_ff_script:new({
	name = "base ball",
	team = Team.kUnassigned,
	model = "models/items/ball/ball.mdl",
	modelskin = 0,
	tosssound = "Flag.Toss",
	dropnotouchtime = 0,
	capnotouchtime = 2,
	hudicon = "hud_ball",
	hudx = 5,
	hudy = 210,
	hudwidth = 48,
	hudheight = 48,
	hudalign = 1, 
	hudstatusiconbluex = 60,
	hudstatusiconbluey = 5,
	hudstatusiconredx = 60,
	hudstatusiconredy = 5,
	hudstatusiconblue = "hud_ball.vtf",
	hudstatusiconred = "hud_ball.vtf",
	hudstatusiconw = 15,
	hudstatusiconh = 15,
	hudstatusiconbluealign = 2,
	hudstatusiconredalign = 3,
	touchflags = {AllowFlags.kOnlyPlayers,AllowFlags.kBlue, AllowFlags.kRed, AllowFlags.kYellow, AllowFlags.kGreen},
	botgoaltype = Bot.kFlag
})

function base_ball:hasanimation() return false end
function base_ball:usephysics() return true end

function base_ball:touch( touch_entity )
	if IsPlayer( touch_entity ) then

		local player = CastToPlayer( touch_entity )
		if self.notouch[player:GetId()] then return end

		local ball = CastToInfoScript( entity )
	
		-- goalies return the ball
		if player:GetTeamId() == Team.kYellow or player:GetTeamId() == Team.kGreen then
			ConsoleToAll( player:GetName() .. " returned the ball!" )
			SmartSound( player, "misc.deeoo", "misc.deeoo", "misc.deeoo" )
			SmartMessage( player, "#FF_WATERPOLO_YOU_RETURN", "#FF_WATERPOLO_TEAM_GOALIE_RETURN", "#FF_WATERPOLO_ENEMY_GOALIE_RETURN" )
			player:AddFortPoints( POINTS_PER_GOALIE_RETURN * 10, "#FF_FORTPOINTS_GOALIE_RETURN" )
			ball:Return()
		-- non-goalies pick up the ball
		-- else
		-- unless feigned?
		elseif not player:IsFeigned() then
			ConsoleToAll( player:GetName() .. " has the ball!" )
			SmartSound( player, "misc.bloop", "misc.bloop", "misc.bloop" )
			SmartMessage( player, "#FF_WATERPOLO_YOU_PICKUP", "#FF_WATERPOLO_TEAM_PICKUP", "#FF_WATERPOLO_ENEMY_PICKUP" )
			ball:Pickup( player )

			AddHudIcon( player, self.hudicon, ball:GetName(), self.hudx, self.hudy, self.hudwidth, self.hudheight, self.hudalign )
			local team = player:GetTeamId()
			if (team == Team.kBlue) then
				AddHudIconToAll( self.hudstatusiconblue, "ball-icon-blue", self.hudstatusiconbluex, self.hudstatusiconbluey, self.hudstatusiconw, self.hudstatusiconh, self.hudstatusiconbluealign )
			elseif (team == Team.kRed) then
				AddHudIconToAll( self.hudstatusiconred, "ball-icon-red", self.hudstatusiconredx, self.hudstatusiconredy, self.hudstatusiconw, self.hudstatusiconh, self.hudstatusiconredalign )
			end
		end
	end
end

function base_ball:onownerdie( owner_entity )
	if IsPlayer( owner_entity ) then
		local player = CastToPlayer( owner_entity )
		-- drop the ball
		ConsoleToAll( player:GetName() .. " died and dropped the ball!" )
		local ball = CastToInfoScript( entity )
		ball:Drop(BALL_RETURN_TIME, 0.0)
		remove_hud_items(ball, player)
	end
end

function base_ball:ownerfeign( owner_entity )
	if IsPlayer( owner_entity ) then
		local player = CastToPlayer( owner_entity )
		-- drop the ball
		ConsoleToAll( player:GetName() .. " feigned and dropped the ball!" )
		local ball = CastToInfoScript( entity )
		ball:Drop(BALL_RETURN_TIME, 0.0)
		remove_hud_items(ball, player)
	end
end

function base_ball:dropitemcmd( drop_entity )
	if IsPlayer( drop_entity ) then
		-- throw the ball
		local ball = CastToInfoScript( entity )

		local player = CastToPlayer( drop_entity )
		if not player:IsInUse() then
			ball:Drop(BALL_RETURN_TIME, BALL_THROW_SPEED)
			ConsoleToAll( player:GetName() .. " passed the ball!" )
		else
			-- ball:Drop(BALL_RETURN_TIME, BALL_THROW_SPEED * 2)
			-- speed actually seems to be capped or somehtin' on the C++ side, because watch this ball not go fast at * 100...
			ball:Drop(BALL_RETURN_TIME, BALL_THROW_SPEED * 100)
			ConsoleToAll( player:GetName() .. " passed the ball really far!" )
		end

		remove_hud_items(ball, player)
	
		SmartSound( player, "misc.woop", "misc.woop", "misc.woop" )
	
		-- Make it so the player can't touch the ball for 1 second
		-- (so it can't be thrown and not stick to the player)
		self:addnotouch(player:GetId(), 1)
	end
end

function base_ball:ondrop( owner_entity )
	if IsPlayer( owner_entity ) then
		local player = CastToPlayer( owner_entity )
		-- let the teams know that the flag was dropped
		SmartSound(player, "yourteam.drop", "yourteam.drop", "otherteam.drop")
		SmartMessage(player, "#FF_YOUBALLDROP", "#FF_TEAMBALLDROP", "#FF_ENEMYBALLDROP")
	end
	
	local ball = CastToInfoScript(entity)
	ball:EmitSound(self.tosssound)
end

function base_ball:onloseitem( owner_entity )
	if IsPlayer( owner_entity ) then
		-- let the player that lost the ball put on a disguise
		local player = CastToPlayer( owner_entity )
		player:SetDisguisable(true)
		player:SetCloakable( true )
	
		self:addnotouch(player:GetId(), self.capnotouchtime)
	end
end

function base_ball:spawn()
	self.notouch = { }
	info_ff_script.spawn(self)
end

function base_ball:addnotouch( player_id, duration )
	self.notouch[player_id] = duration
	AddSchedule(self.name .. "-" .. player_id, duration, self.removenotouch, self, player_id)
end

function base_ball.removenotouch(self, player_id)
	self.notouch[player_id] = nil
end

-- For when this object is carried, these offsets are used to place
-- the info_ff_script relative to the players feet
function base_ball:attachoffset()
	-- x = forward/backward
	-- y = left/right
	-- z = up/down
	local offset = Vector( 32, 0, 0 )
	return offset
end

function base_ball:precache()
	PrecacheSound(self.tosssound)
	info_ff_script.precache(self)
end

function base_ball:onreturn()
	ConsoleToAll( "The ball has returned!" )
	BroadCastSound ( "misc.deeoo" )
	BroadCastMessage( "#FF_WATERPOLO_BALL_RETURN" )
end

ball = base_ball:new({
	touchflags = {AllowFlags.kOnlyPlayers,AllowFlags.kBlue, AllowFlags.kRed, AllowFlags.kYellow, AllowFlags.kGreen},
	botgoaltype = Bot.kFlag
})




-- generic goal
base_goal = trigger_ff_script:new({
	health = 100,
	armor = 300,
	grenades = 200,
	nails = 200,
	shells = 200,
	rockets = 200,
	cells = 200,
	detpacks = 1,
	gren1 = 4,
	gren2 = 4,
	item = "",
	team = 0,
	botgoaltype = Bot.kFlagCap
})

function base_goal:allowed ( allowed_entity )
	if IsPlayer( allowed_entity ) then		
		-- player has to be carrying the item
		local player = CastToPlayer( allowed_entity )
		return player:HasItem( self.item )
	end

	return EVENT_DISALLOWED
end

function base_goal:ontrigger ( trigger_entity )
	if IsPlayer( trigger_entity ) then

		-- player should capture now
		local player = CastToPlayer( trigger_entity )
		local team = GetTeam( self.team )
		team:AddScore( POINTS_PER_CAPTURE )
		ConsoleToAll( team:GetName() .. " team scores!" )

		local ball = GetInfoScriptByName( self.item )
		ball:Return()

		remove_hud_items(ball, player)

		if player:GetTeamId() == self.team then
			ConsoleToAll( player:GetName() .. " scored a goal!" )
			SmartSound( player, "misc.doop", "misc.doop", "misc.dadeda" )
			SmartMessage( player, "#FF_WATERPOLO_YOU_GOAL", "#FF_WATERPOLO_TEAM_GOAL", "#FF_WATERPOLO_ENEMY_GOAL" )
			player:AddFortPoints( POINTS_PER_CAPTURE * 10, "#FF_FORTPOINTS_GOAL" )
		else
			ConsoleToAll( player:GetName() .. " scored an own goal!\nWait, what?!  Kill him like that Columbian a few years ago!  OWN GOAL!  OWN GOAL!" )
			SmartSound( player, "misc.dadeda", "misc.dadeda", "misc.doop" )
			SmartMessage( player, "#FF_WATERPOLO_YOU_OWN_GOAL", "#FF_WATERPOLO_TEAM_OWN_GOAL", "#FF_WATERPOLO_ENEMY_OWN_GOAL" )
			player:AddFortPoints( -POINTS_PER_CAPTURE * 10, "#FF_FORTPOINTS_OWN_GOAL" )
		end

		ApplyToAll({ AT.kRemovePacks, AT.kRemoveProjectiles, AT.kRespawnPlayers, AT.kRemoveBuildables, AT.kRemoveRagdolls, AT.kStopPrimedGrens, AT.kReloadClips })
	end
end

-- on blue's side of map, but points go to red
blue_goal = base_goal:new({ item = "ball", team = Team.kRed })

-- on red's side of map, but points go to blue
red_goal = base_goal:new({ item = "ball", team = Team.kBlue })




-- returns the item
base_item_returner = trigger_ff_script:new({
	item = "",
	message = ""
})

function base_item_returner:allowed ( allowed_entity )

	if IsPlayer( allowed_entity ) then
		-- player has to be carrying the item
		local player = CastToPlayer( allowed_entity )
		return player:HasItem( self.item )
	end

	return EVENT_ALLOWED

end

function base_item_returner:ontrigger ( trigger_entity )

	if IsPlayer( trigger_entity ) then
		local player = CastToPlayer( trigger_entity )
		-- return the item
		local item = GetInfoScriptByName( self.item )
		item:Return()
		remove_hud_items(item, player)
		ConsoleToAll( player:GetName() .. self.message )
		SmartSound( player, "misc.deeoo", "misc.deeoo", "misc.deeoo" )
		SmartMessage( player, "#FF_WATERPOLO_YOU_BOUNDS", "#FF_WATERPOLO_TEAM_BOUNDS", "#FF_WATERPOLO_ENEMY_BOUNDS" )
	end

end

-- ball stripper
ball_stripper = base_item_returner:new({ item = "ball", message = " took the ball out of bounds!" })




-- goalie trigger_teleports
trigger_teleport = trigger_ff_script:new({})
base_goalie_teleport = trigger_teleport:new({ team = 0 })
function base_goalie_teleport:allowed( allowed_entity ) 
	if IsPlayer( allowed_entity ) then
		local player = CastToPlayer( allowed_entity )
		return player:GetTeamId() == self.team 
	end

	return EVENT_ALLOWED
end

-- blue goalie is on the yellow team
blue_goalie_teleport = base_goalie_teleport:new({ team = Team.kYellow })

-- red goalie is on the green team
red_goalie_teleport = base_goalie_teleport:new({ team = Team.kGreen })




-- spawn pack
waterpolo_pack_spawn = genericbackpack:new({
	health = 100,
	armor = 300,
	grenades = 200,
	nails = 200,
	shells = 200,
	rockets = 200,
	cells = 200,
	detpacks = 0,
	gren1 = 0,
	gren2 = 0,
	respawntime = 1,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
})

-- grenade pack
waterpolo_pack_grenades = genericbackpack:new({
	health = 100,
	armor = 300,
	grenades = 200,
	nails = 200,
	shells = 200,
	rockets = 200,
	cells = 200,
	detpacks = 1,
	gren1 = 4,
	gren2 = 4,
	respawntime = 30,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
	team = 0,
	notallowedmsg = false,
})

function waterpolo_pack_grenades:dispenseallowed ( allowed_entity )
	if IsPlayer( allowed_entity ) then
		local player = CastToPlayer( allowed_entity )
		if player:GetTeamId() == self.team then
			-- Don't know if grenade packs should have an allowed message, but whatever...
			BroadCastMessageToPlayer( player, "#FF_WATERPOLO_GRENADES" )
			return EVENT_ALLOWED
		else
			-- Could just make notallowedmsg = the custom message,
			-- but notallowedmsg = false is also being tested.
			BroadCastMessageToPlayer( player, "#FF_WATERPOLO_NO_GRENADES_FOR_YOU" )
			return EVENT_DISALLOWED
		end
	end
end

waterpolo_pack_grenades_blue = waterpolo_pack_grenades:new({ team = Team.kBlue })
waterpolo_pack_grenades_red = waterpolo_pack_grenades:new({ team = Team.kRed })

